package com.example.androidmp3;

import android.app.Activity;
import android.app.ListActivity;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaMetadataRetriever;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class AndroidMP3 extends ListActivity 
{
  private static final String MEDIA_PATH = new String("/sdcard/");
  private static final String TAG = "AndroidMP3";
  private List<String> mp3List = new ArrayList<String>();
  private MediaPlayer mp = new MediaPlayer();
  private Button playpauseButton;
  private Button nextButton;
  private Button prevButton;
  private int currentPosition = 0;
  private boolean pause = false;

  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState)
  {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.list);
    setUpButtons();
    updateMP3List();
  }

  public void updateMP3List() {
    mp3List.clear();
    File home = new File(MEDIA_PATH);
    // try block needed because even if there is no SD card, /sdcard/
    // directory exists (so can't test for existence or being a dir)
    try { 
      if ( home.listFiles( new Mp3Filter() ).length > 0 ) {
        for ( File file : home.listFiles( new Mp3Filter() ) ) {
          mp3List.add(file.getName());
        }
        ArrayAdapter<String> mp3ListAdapter = 
            new ArrayAdapter<String>(this, R.layout.item, mp3List);
          setListAdapter(mp3ListAdapter);
        }
      else {
        // no files here; 'empty' string will be seen
      }
    } catch (NullPointerException e) {
      // No SD card; 'empty' string will be seen
      return;
    }
  }

  @Override
  protected void onListItemClick(ListView list, View view, 
                                 int position, long id) {
    currentPosition = position;
    playMp3(MEDIA_PATH + mp3List.get(position));
  }

  @Override
  protected void onStop() {
    mp.release();
    mp = null;
    super.onStop();
  }

  @Override
  protected void onResume() {
    mp = new MediaPlayer();
    updateMP3List();
    super.onResume();
  }

  private void setUpButtons() {
    playpauseButton = (Button)this.findViewById(R.id.playpause);
    nextButton = (Button)this.findViewById(R.id.next);
    prevButton = (Button)this.findViewById(R.id.prev);

    playpauseButton.setOnClickListener(new OnClickListener() {
      @Override
      public void onClick(View v) {
        if (mp.isPlaying()) { 
          pauseMp3();
        } else {
          playMp3(MEDIA_PATH + mp3List.get(currentPosition));
        }
      }
    });

    nextButton.setOnClickListener(new OnClickListener() {
      @Override
      public void onClick(View v) {
        nextMp3();
      }
    });

    prevButton.setOnClickListener(new OnClickListener() {
      @Override
      public void onClick(View v) {
        prevMp3();
      }
    });
  }

  private void playMp3(String mp3Path) {
    try {
      if (pause == false) {
        mp.reset();
        mp.setDataSource(mp3Path);
        mp.prepare();
        setTrackInfo(mp3Path);
      }
      mp.start();
      pause = false;

      // Move to next song automatically
      mp.setOnCompletionListener(new OnCompletionListener() {
        public void onCompletion(MediaPlayer mplayer) {
          nextMp3();
        }
      });
    } catch (IOException e) {
      Log.v(TAG, e.getMessage());
    }
  }

  private void pauseMp3() {
    mp.pause();
    pause = true;
  }

  private void nextMp3() {
    if (++currentPosition >= mp3List.size()) {
      // No more MP3s! Reset currentPosition to start of list
      currentPosition = 0;
    } else {
      pause = false;
      playMp3(MEDIA_PATH + mp3List.get(currentPosition));
    }
  }

  private void prevMp3() {
    if (--currentPosition < 0) {
      // No more MP3s! Reset currentPosition to start of list
      currentPosition = 0;
    } else {
      pause = false;
      playMp3(MEDIA_PATH + mp3List.get(currentPosition));
    }
  }

  private void setTrackInfo(String mp3Path) {
    TextView currentmp3 = (TextView)this.findViewById(R.id.currentmp3);
    MediaMetadataRetriever metadata = new MediaMetadataRetriever();
    metadata.setDataSource(mp3Path);
    String artist = 
        metadata.extractMetadata(MediaMetadataRetriever.METADATA_KEY_ARTIST);
    String title = 
        metadata.extractMetadata(MediaMetadataRetriever.METADATA_KEY_TITLE);
    currentmp3.setText(title + " - " + artist);
    metadata.release();
  }

  class Mp3Filter implements FilenameFilter {
    public boolean accept(File dir, String name) {
      return (name.endsWith(".mp3"));
    }
  }
}
